#include <iostream>
#include <string>

#include "rateconn.H"
#include "Mutex.H"


using namespace std;

RateconnTable *rateconnTable = NULL;

extern "C" void rateconnInitWrapper(pthread_mutex_t *mutex, int numOfBuckets) throw () {
    rateconnTable = new RateconnTable(mutex, numOfBuckets);
}

extern "C" void rateconnTurnWrapper() throw () {
    rateconnTable->turn();
}

extern "C" int rateconnIncAndGetWrapper(const char* input) throw () {
    return rateconnTable->incAndGet(string(input));
}



/**
- a certain number of hashes, sitting on a kind of a chain
- periodically the chain is moved a step forward, the last hash while be dropped
  off from the end of the chain and wiped out, a new one is create and put at the
  first position of the chain
- new entries a put into the first one
- all hashes are considered when calculating the number of events in a timespan
- 



*/

RateconnTable::RateconnTable(pthread_mutex_t *mutex, int numOfBuckets) 
    : m_numOfBuckets(numOfBuckets), m_mutex(mutex)
{
    turn();
}

void RateconnTable::turn() {
    Mutex m1(m_mutex); // rmc to lock the whole scope and to unlock it at its end
    
    if (m_countersDeque.size() > m_numOfBuckets)
        m_countersDeque.pop_back();
    m_countersDeque.push_front(counterMap_t());
}

void RateconnTable::printAll() {
    int t = 0;
    for (countersDeque_t::const_iterator i = m_countersDeque.begin(); i != m_countersDeque.end(); ++i, ++t) {
        for (counterMap_t::const_iterator j = i->begin(); j != i->end(); ++j) {
            cout << "table: " << t <<", key: " << j->first << ", value: " << j->second << endl;
        }
    }
}

int RateconnTable::incAndGet(const string& input) {
    Mutex m1(m_mutex); // rmc to lock the whole scope and to unlock it at its end

    counterMap_t& cm = m_countersDeque.front();
    if (cm.find(input) == 0)
        cm[input] = 0;
    cm[input]++;

    int cnt = 0;
    for (countersDeque_t::iterator i = m_countersDeque.begin(); i != m_countersDeque.end(); ++i) {
        cnt += (*i)[input];
    }
    
    printAll();
    
    return cnt;
}


